@@ -37,19 +37,12 @@
 #include "nsIProtocolHandler.h"
 #include "imgIRequest.h"
 
+#include "mozilla/IntegerPrintfMacros.h"
+
 using namespace mozilla;
 using namespace mozilla::image;
 
-PRLogModuleInfo*
-GetImgLog()
-{
-  static PRLogModuleInfo* sImgLog;
-  if (!sImgLog) {
-    sImgLog = PR_NewLogModule("imgRequest");
-  }
-  return sImgLog;
-}
-#define LOG_TEST(level) (GetImgLog() && MOZ_LOG_TEST(GetImgLog(), (level)))
+#define LOG_TEST(level) (MOZ_LOG_TEST(gImgLog, (level)))
 
 NS_IMPL_ISUPPORTS(imgRequest,
                   nsIStreamListener, nsIRequestObserver,
@@ -66,7 +59,7 @@ imgRequest::imgRequest(imgLoader* aLoader, const ImageCacheKey& aCacheKey)
  , mValidator(nullptr)
  , mInnerWindowId(0)
  , mCORSMode(imgIRequest::CORS_NONE)
- , mReferrerPolicy(mozilla::net::RP_Default)
+ , mReferrerPolicy(mozilla::net::RP_Unset)
  , mImageErrorCode(NS_OK)
  , mMutex("imgRequest")
  , mProgressTracker(new ProgressTracker())
@@ -86,10 +79,10 @@ imgRequest::~imgRequest()
   if (mURI) {
     nsAutoCString spec;
     mURI->GetSpec(spec);
-    LOG_FUNC_WITH_PARAM(GetImgLog(), "imgRequest::~imgRequest()",
+    LOG_FUNC_WITH_PARAM(gImgLog, "imgRequest::~imgRequest()",
                         "keyuri", spec.get());
   } else
-    LOG_FUNC(GetImgLog(), "imgRequest::~imgRequest()");
+    LOG_FUNC(gImgLog, "imgRequest::~imgRequest()");
 }
 
 nsresult
@@ -106,7 +99,7 @@ imgRequest::Init(nsIURI *aURI,
 {
   MOZ_ASSERT(NS_IsMainThread(), "Cannot use nsIURI off main thread!");
 
-  LOG_FUNC(GetImgLog(), "imgRequest::Init");
+  LOG_FUNC(gImgLog, "imgRequest::Init");
 
   MOZ_ASSERT(!mImage, "Multiple calls to init");
   MOZ_ASSERT(aURI, "No uri");
@@ -117,7 +110,10 @@ imgRequest::Init(nsIURI *aURI,
   mProperties = do_CreateInstance("@mozilla.org/properties;1");
 
   // Use ImageURL to ensure access to URI data off main thread.
-  mURI = new ImageURL(aURI);
+  nsresult rv;
+  mURI = new ImageURL(aURI, rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
   mCurrentURI = aCurrentURI;
   mRequest = aRequest;
   mChannel = aChannel;
@@ -184,16 +180,16 @@ imgRequest::GetProgressTracker() const
     MOZ_ASSERT(!mProgressTracker,
                "Should have given mProgressTracker to mImage");
     return mImage->GetProgressTracker();
-  } else {
-    MOZ_ASSERT(mProgressTracker,
-               "Should have mProgressTracker until we create mImage");
-    RefPtr<ProgressTracker> progressTracker = mProgressTracker;
-    MOZ_ASSERT(progressTracker);
-    return progressTracker.forget();
   }
+  MOZ_ASSERT(mProgressTracker,
+             "Should have mProgressTracker until we create mImage");
+  RefPtr<ProgressTracker> progressTracker = mProgressTracker;
+  MOZ_ASSERT(progressTracker);
+  return progressTracker.forget();
 }
 
-void imgRequest::SetCacheEntry(imgCacheEntry* entry)
+void
+imgRequest::SetCacheEntry(imgCacheEntry* entry)
 {
   mCacheEntry = entry;
 }
@@ -216,7 +212,7 @@ void
 imgRequest::AddProxy(imgRequestProxy* proxy)
 {
   NS_PRECONDITION(proxy, "null imgRequestProxy passed in");
-  LOG_SCOPE_WITH_PARAM(GetImgLog(), "imgRequest::AddProxy", "proxy", proxy);
+  LOG_SCOPE_WITH_PARAM(gImgLog, "imgRequest::AddProxy", "proxy", proxy);
 
   if (!mFirstProxy) {
     // Save a raw pointer to the first proxy we see, for use in the network
@@ -240,7 +236,7 @@ imgRequest::AddProxy(imgRequestProxy* proxy)
 nsresult
 imgRequest::RemoveProxy(imgRequestProxy* proxy, nsresult aStatus)
 {
-  LOG_SCOPE_WITH_PARAM(GetImgLog(), "imgRequest::RemoveProxy", "proxy", proxy);
+  LOG_SCOPE_WITH_PARAM(gImgLog, "imgRequest::RemoveProxy", "proxy", proxy);
 
   // This will remove our animation consumers, so after removing
   // this proxy, we don't end up without proxies with observers, but still
@@ -266,10 +262,10 @@ imgRequest::RemoveProxy(imgRequestProxy* proxy, nsresult aStatus)
       if (mLoader) {
         mLoader->SetHasNoProxies(this, mCacheEntry);
       }
-    } else if (MOZ_LOG_TEST(GetImgLog(), LogLevel::Debug)) {
+    } else if (MOZ_LOG_TEST(gImgLog, LogLevel::Debug)) {
       nsAutoCString spec;
       mURI->GetSpec(spec);
-      LOG_MSG_WITH_PARAM(GetImgLog(),
+      LOG_MSG_WITH_PARAM(gImgLog,
                          "imgRequest::RemoveProxy no cache entry",
                          "uri", spec.get());
     }
@@ -282,7 +278,7 @@ imgRequest::RemoveProxy(imgRequestProxy* proxy, nsresult aStatus)
      */
     if (!(progressTracker->GetProgress() & FLAG_LAST_PART_COMPLETE) &&
         NS_FAILED(aStatus)) {
-      LOG_MSG(GetImgLog(), "imgRequest::RemoveProxy",
+      LOG_MSG(gImgLog, "imgRequest::RemoveProxy",
               "load in progress.  canceling");
 
       this->Cancel(NS_BINDING_ABORTED);
@@ -304,7 +300,7 @@ imgRequest::RemoveProxy(imgRequestProxy* proxy, nsresult aStatus)
 void
 imgRequest::CancelAndAbort(nsresult aStatus)
 {
-  LOG_SCOPE(GetImgLog(), "imgRequest::CancelAndAbort");
+  LOG_SCOPE(gImgLog, "imgRequest::CancelAndAbort");
 
   Cancel(aStatus);
 
@@ -317,7 +313,7 @@ imgRequest::CancelAndAbort(nsresult aStatus)
   }
 }
 
-class imgRequestMainThreadCancel : public nsRunnable
+class imgRequestMainThreadCancel : public Runnable
 {
 public:
   imgRequestMainThreadCancel(imgRequest* aImgRequest, nsresult aStatus)
@@ -328,7 +324,7 @@ public:
     MOZ_ASSERT(aImgRequest);
   }
 
-  NS_IMETHOD Run()
+  NS_IMETHOD Run() override
   {
     MOZ_ASSERT(NS_IsMainThread(), "I should be running on the main thread!");
     mImgRequest->ContinueCancel(mStatus);
@@ -343,7 +339,7 @@ void
 imgRequest::Cancel(nsresult aStatus)
 {
   /* The Cancel() method here should only be called by this class. */
-  LOG_SCOPE(GetImgLog(), "imgRequest::Cancel");
+  LOG_SCOPE(gImgLog, "imgRequest::Cancel");
 
   if (NS_IsMainThread()) {
     ContinueCancel(aStatus);
@@ -367,7 +363,7 @@ imgRequest::ContinueCancel(nsresult aStatus)
   }
 }
 
-class imgRequestMainThreadEvict : public nsRunnable
+class imgRequestMainThreadEvict : public Runnable
 {
 public:
   explicit imgRequestMainThreadEvict(imgRequest* aImgRequest)
@@ -377,7 +373,7 @@ public:
     MOZ_ASSERT(aImgRequest);
   }
 
-  NS_IMETHOD Run()
+  NS_IMETHOD Run() override
   {
     MOZ_ASSERT(NS_IsMainThread(), "I should be running on the main thread!");
     mImgRequest->ContinueEvict();
@@ -392,7 +388,7 @@ void
 imgRequest::EvictFromCache()
 {
   /* The EvictFromCache() method here should only be called by this class. */
-  LOG_SCOPE(GetImgLog(), "imgRequest::EvictFromCache");
+  LOG_SCOPE(gImgLog, "imgRequest::EvictFromCache");
 
   if (NS_IsMainThread()) {
     ContinueEvict();
@@ -411,7 +407,7 @@ imgRequest::ContinueEvict()
 }
 
 void
-imgRequest::RequestDecode()
+imgRequest::StartDecoding()
 {
   MutexAutoLock lock(mMutex);
   mDecodeRequested = true;
@@ -428,7 +424,7 @@ nsresult imgRequest::GetURI(ImageURL** aURI)
 {
   MOZ_ASSERT(aURI);
 
-  LOG_FUNC(GetImgLog(), "imgRequest::GetURI");
+  LOG_FUNC(gImgLog, "imgRequest::GetURI");
 
   if (mURI) {
     *aURI = mURI;
@@ -444,7 +440,7 @@ imgRequest::GetCurrentURI(nsIURI** aURI)
 {
   MOZ_ASSERT(aURI);
 
-  LOG_FUNC(GetImgLog(), "imgRequest::GetCurrentURI");
+  LOG_FUNC(gImgLog, "imgRequest::GetCurrentURI");
 
   if (mCurrentURI) {
     *aURI = mCurrentURI;
@@ -474,7 +470,7 @@ imgRequest::GetImageErrorCode()
 nsresult
 imgRequest::GetSecurityInfo(nsISupports** aSecurityInfo)
 {
-  LOG_FUNC(GetImgLog(), "imgRequest::GetSecurityInfo");
+  LOG_FUNC(gImgLog, "imgRequest::GetSecurityInfo");
 
   // Missing security info means this is not a security load
   // i.e. it is not an error when security info is missing
@@ -485,7 +481,7 @@ imgRequest::GetSecurityInfo(nsISupports** aSecurityInfo)
 void
 imgRequest::RemoveFromCache()
 {
-  LOG_SCOPE(GetImgLog(), "imgRequest::RemoveFromCache");
+  LOG_SCOPE(gImgLog, "imgRequest::RemoveFromCache");
 
   bool isInCache = false;
 
@@ -561,7 +557,7 @@ imgRequest::HasTransferredData() const
 void
 imgRequest::SetIsInCache(bool aInCache)
 {
-  LOG_FUNC_WITH_PARAM(GetImgLog(),
+  LOG_FUNC_WITH_PARAM(gImgLog,
                       "imgRequest::SetIsCacheable", "aInCache", aInCache);
   MutexAutoLock lock(mMutex);
   mIsInCache = aInCache;
@@ -711,7 +707,7 @@ imgRequest::HadInsecureRedirect() const
 NS_IMETHODIMP
 imgRequest::OnStartRequest(nsIRequest* aRequest, nsISupports* ctxt)
 {
-  LOG_SCOPE(GetImgLog(), "imgRequest::OnStartRequest");
+  LOG_SCOPE(gImgLog, "imgRequest::OnStartRequest");
 
   RefPtr<Image> image;
 
@@ -786,10 +782,10 @@ imgRequest::OnStartRequest(nsIRequest* aRequest, nsISupports* ctxt)
         DecodePool::Singleton()->GetIOEventTarget();
       rv = retargetable->RetargetDeliveryTo(target);
     }
-    MOZ_LOG(GetImgLog(), LogLevel::Warning,
+    MOZ_LOG(gImgLog, LogLevel::Warning,
            ("[this=%p] imgRequest::OnStartRequest -- "
-            "RetargetDeliveryTo rv %d=%s\n",
-            this, rv, NS_SUCCEEDED(rv) ? "succeeded" : "failed"));
+            "RetargetDeliveryTo rv %" PRIu32 "=%s\n",
+            this, static_cast<uint32_t>(rv), NS_SUCCEEDED(rv) ? "succeeded" : "failed"));
   }
 
   return NS_OK;
@@ -799,11 +795,17 @@ NS_IMETHODIMP
 imgRequest::OnStopRequest(nsIRequest* aRequest,
                           nsISupports* ctxt, nsresult status)
 {
-  LOG_FUNC(GetImgLog(), "imgRequest::OnStopRequest");
+  LOG_FUNC(gImgLog, "imgRequest::OnStopRequest");
   MOZ_ASSERT(NS_IsMainThread(), "Can't send notifications off-main-thread");
 
   RefPtr<Image> image = GetImage();
 
+  RefPtr<imgRequest> strongThis = this;
+
+  if (mIsMultiPartChannel && mNewPartPending) {
+    OnDataAvailable(aRequest, ctxt, nullptr, 0, 0);
+  }
+
   // XXXldb What if this is a non-last part of a multipart request?
   // xxx before we release our reference to mRequest, lets
   // save the last status that we saw so that the
@@ -885,7 +887,7 @@ struct mimetype_closure
 };
 
 /* prototype for these defined below */
-static NS_METHOD
+static nsresult
 sniff_mimetype_callback(nsIInputStream* in, void* closure,
                         const char* fromRawSegment, uint32_t toOffset,
                         uint32_t count, uint32_t* writeCount);
@@ -925,23 +927,27 @@ PrepareForNewPart(nsIRequest* aRequest, nsIInputStream* aInStr, uint32_t aCount,
 {
   NewPartResult result(aExistingImage);
 
-  mimetype_closure closure;
-  closure.newType = &result.mContentType;
+  if (aInStr) {
+    mimetype_closure closure;
+    closure.newType = &result.mContentType;
 
-  // Look at the first few bytes and see if we can tell what the data is from
-  // that since servers tend to lie. :(
-  uint32_t out;
-  aInStr->ReadSegments(sniff_mimetype_callback, &closure, aCount, &out);
+    // Look at the first few bytes and see if we can tell what the data is from
+    // that since servers tend to lie. :(
+    uint32_t out;
+    aInStr->ReadSegments(sniff_mimetype_callback, &closure, aCount, &out);
+  }
 
   nsCOMPtr<nsIChannel> chan(do_QueryInterface(aRequest));
   if (result.mContentType.IsEmpty()) {
     nsresult rv = chan ? chan->GetContentType(result.mContentType)
                        : NS_ERROR_FAILURE;
     if (NS_FAILED(rv)) {
-      MOZ_LOG(GetImgLog(),
+      MOZ_LOG(gImgLog,
               LogLevel::Error, ("imgRequest::PrepareForNewPart -- "
                                 "Content type unavailable from the channel\n"));
-      return result;
+      if (!aIsMultipart) {
+        return result;
+      }
     }
   }
 
@@ -949,7 +955,7 @@ PrepareForNewPart(nsIRequest* aRequest, nsIInputStream* aInStr, uint32_t aCount,
     chan->GetContentDispositionHeader(result.mContentDisposition);
   }
 
-  MOZ_LOG(GetImgLog(), LogLevel::Debug,
+  MOZ_LOG(gImgLog, LogLevel::Debug,
          ("imgRequest::PrepareForNewPart -- Got content type %s\n",
           result.mContentType.get()));
 
@@ -969,6 +975,7 @@ PrepareForNewPart(nsIRequest* aRequest, nsIInputStream* aInStr, uint32_t aCount,
     if (result.mIsFirstPart) {
       // First part for a multipart channel. Create the MultipartImage wrapper.
       MOZ_ASSERT(aProgressTracker, "Shouldn't have given away tracker yet");
+      aProgressTracker->SetIsMultipart();
       result.mImage =
         ImageFactory::CreateMultipartImage(partImage, aProgressTracker);
     } else {
@@ -1001,7 +1008,7 @@ PrepareForNewPart(nsIRequest* aRequest, nsIInputStream* aInStr, uint32_t aCount,
   return result;
 }
 
-class FinishPreparingForNewPartRunnable final : public nsRunnable
+class FinishPreparingForNewPartRunnable final : public Runnable
 {
 public:
   FinishPreparingForNewPartRunnable(imgRequest* aImgRequest,
@@ -1044,7 +1051,7 @@ imgRequest::FinishPreparingForNewPart(const NewPartResult& aResult)
   }
 
   if (IsDecodeRequested()) {
-    aResult.mImage->RequestDecode();
+    aResult.mImage->StartDecoding(imgIContainer::FLAG_NONE);
   }
 }
 
@@ -1053,7 +1060,7 @@ imgRequest::OnDataAvailable(nsIRequest* aRequest, nsISupports* aContext,
                             nsIInputStream* aInStr, uint64_t aOffset,
                             uint32_t aCount)
 {
-  LOG_SCOPE_WITH_PARAM(GetImgLog(), "imgRequest::OnDataAvailable",
+  LOG_SCOPE_WITH_PARAM(gImgLog, "imgRequest::OnDataAvailable",
                        "count", aCount);
 
   NS_ASSERTION(aRequest, "imgRequest::OnDataAvailable -- no request!");
@@ -1111,15 +1118,17 @@ imgRequest::OnDataAvailable(nsIRequest* aRequest, nsISupports* aContext,
   }
 
   // Notify the image that it has new data.
-  nsresult rv =
-    image->OnImageDataAvailable(aRequest, aContext, aInStr, aOffset, aCount);
+  if (aInStr) {
+    nsresult rv =
+      image->OnImageDataAvailable(aRequest, aContext, aInStr, aOffset, aCount);
 
-  if (NS_FAILED(rv)) {
-    MOZ_LOG(GetImgLog(), LogLevel::Warning,
-           ("[this=%p] imgRequest::OnDataAvailable -- "
-            "copy to RasterImage failed\n", this));
-    Cancel(NS_IMAGELIB_ERROR_FAILURE);
-    return NS_BINDING_ABORTED;
+    if (NS_FAILED(rv)) {
+      MOZ_LOG(gImgLog, LogLevel::Warning,
+             ("[this=%p] imgRequest::OnDataAvailable -- "
+              "copy to RasterImage failed\n", this));
+      Cancel(NS_IMAGELIB_ERROR_FAILURE);
+      return NS_BINDING_ABORTED;
+    }
   }
 
   return NS_OK;
@@ -1148,7 +1157,7 @@ imgRequest::SetProperties(const nsACString& aContentType,
   }
 }
 
-static NS_METHOD
+static nsresult
 sniff_mimetype_callback(nsIInputStream* in,
                         void* data,
                         const char* fromRawSegment,
@@ -1234,12 +1243,10 @@ imgRequest::OnRedirectVerifyCallback(nsresult result)
   mNewRedirectChannel = nullptr;
 
   if (LOG_TEST(LogLevel::Debug)) {
-    nsAutoCString spec;
-    if (mCurrentURI) {
-      mCurrentURI->GetSpec(spec);
-    }
-    LOG_MSG_WITH_PARAM(GetImgLog(),
-                       "imgRequest::OnChannelRedirect", "old", spec.get());
+    LOG_MSG_WITH_PARAM(gImgLog,
+                       "imgRequest::OnChannelRedirect", "old",
+                       mCurrentURI ? mCurrentURI->GetSpecOrDefault().get()
+                                   : "");
   }
 
   // If the previous URI is a non-HTTPS URI, record that fact for later use by
@@ -1272,12 +1279,9 @@ imgRequest::OnRedirectVerifyCallback(nsresult result)
   mChannel->GetURI(getter_AddRefs(mCurrentURI));
 
   if (LOG_TEST(LogLevel::Debug)) {
-    nsAutoCString spec;
-    if (mCurrentURI) {
-      mCurrentURI->GetSpec(spec);
-    }
-    LOG_MSG_WITH_PARAM(GetImgLog(), "imgRequest::OnChannelRedirect",
-                       "new", spec.get());
+    LOG_MSG_WITH_PARAM(gImgLog, "imgRequest::OnChannelRedirect", "new",
+                       mCurrentURI ? mCurrentURI->GetSpecOrDefault().get()
+                                   : "");
   }
 
   // Make sure we have a protocol that returns data rather than opens an
