@@ -23,7 +23,7 @@ using namespace mozilla::image;
 class ProxyBehaviour
 {
  public:
-  virtual ~ProxyBehaviour() {}
+  virtual ~ProxyBehaviour() = default;
 
   virtual already_AddRefed<mozilla::image::Image> GetImage() const = 0;
   virtual bool HasImage() const = 0;
@@ -37,15 +37,15 @@ class RequestBehaviour : public ProxyBehaviour
  public:
   RequestBehaviour() : mOwner(nullptr), mOwnerHasImage(false) {}
 
-  virtual already_AddRefed<mozilla::image::Image>GetImage() const override;
-  virtual bool HasImage() const override;
-  virtual already_AddRefed<ProgressTracker> GetProgressTracker() const override;
+  already_AddRefed<mozilla::image::Image>GetImage() const override;
+  bool HasImage() const override;
+  already_AddRefed<ProgressTracker> GetProgressTracker() const override;
 
-  virtual imgRequest* GetOwner() const override {
+  imgRequest* GetOwner() const override {
     return mOwner;
   }
 
-  virtual void SetOwner(imgRequest* aOwner) override {
+  void SetOwner(imgRequest* aOwner) override {
     mOwner = aOwner;
 
     if (mOwner) {
@@ -160,7 +160,7 @@ imgRequestProxy::Init(imgRequest* aOwner,
   NS_PRECONDITION(!GetOwner() && !mListener,
                   "imgRequestProxy is already initialized");
 
-  LOG_SCOPE_WITH_PARAM(GetImgLog(), "imgRequestProxy::Init", "request",
+  LOG_SCOPE_WITH_PARAM(gImgLog, "imgRequestProxy::Init", "request",
                        aOwner);
 
   MOZ_ASSERT(mAnimationConsumers == 0, "Cannot have animation before Init");
@@ -208,15 +208,6 @@ imgRequestProxy::ChangeOwner(imgRequest* aNewOwner)
   uint32_t oldAnimationConsumers = mAnimationConsumers;
   ClearAnimationConsumers();
 
-  // Were we decoded before?
-  bool wasDecoded = false;
-  RefPtr<ProgressTracker> progressTracker = GetProgressTracker();
-  if (progressTracker->HasImage() &&
-      progressTracker->GetImageStatus() &
-        imgIRequest::STATUS_FRAME_COMPLETE) {
-    wasDecoded = true;
-  }
-
   GetOwner()->RemoveProxy(this, NS_IMAGELIB_CHANGING_OWNER);
 
   mBehaviour->SetOwner(aNewOwner);
@@ -235,10 +226,10 @@ imgRequestProxy::ChangeOwner(imgRequest* aNewOwner)
 
   GetOwner()->AddProxy(this);
 
-  // If we were decoded, or if we'd previously requested a decode, request a
-  // decode on the new image
-  if (wasDecoded || mDecodeRequested) {
-    StartDecoding();
+  // If we'd previously requested a synchronous decode, request a decode on the
+  // new image.
+  if (mDecodeRequested) {
+    StartDecoding(imgIContainer::FLAG_NONE);
   }
 
   return NS_OK;
@@ -312,7 +303,7 @@ imgRequestProxy::Cancel(nsresult status)
     return NS_ERROR_FAILURE;
   }
 
-  LOG_SCOPE(GetImgLog(), "imgRequestProxy::Cancel");
+  LOG_SCOPE(gImgLog, "imgRequestProxy::Cancel");
 
   mCanceled = true;
 
@@ -343,7 +334,7 @@ imgRequestProxy::CancelAndForgetObserver(nsresult aStatus)
     return NS_ERROR_FAILURE;
   }
 
-  LOG_SCOPE(GetImgLog(), "imgRequestProxy::CancelAndForgetObserver");
+  LOG_SCOPE(gImgLog, "imgRequestProxy::CancelAndForgetObserver");
 
   mCanceled = true;
 
@@ -358,9 +349,8 @@ imgRequestProxy::CancelAndForgetObserver(nsresult aStatus)
   mIsInLoadGroup = oldIsInLoadGroup;
 
   if (mIsInLoadGroup) {
-    nsCOMPtr<nsIRunnable> ev =
-      NS_NewRunnableMethod(this, &imgRequestProxy::DoRemoveFromLoadGroup);
-    NS_DispatchToCurrentThread(ev);
+    NS_DispatchToCurrentThread(NewRunnableMethod("imgRequestProxy::DoRemoveFromLoadGroup",
+                                                 this, &imgRequestProxy::DoRemoveFromLoadGroup));
   }
 
   NullOutListener();
@@ -369,42 +359,41 @@ imgRequestProxy::CancelAndForgetObserver(nsresult aStatus)
 }
 
 NS_IMETHODIMP
-imgRequestProxy::StartDecoding()
+imgRequestProxy::StartDecoding(uint32_t aFlags)
 {
   // Flag this, so we know to transfer the request if our owner changes
   mDecodeRequested = true;
 
   RefPtr<Image> image = GetImage();
   if (image) {
-    return image->StartDecoding();
+    return image->StartDecoding(aFlags);
   }
 
   if (GetOwner()) {
-    GetOwner()->RequestDecode();
+    GetOwner()->StartDecoding();
   }
 
   return NS_OK;
 }
 
-NS_IMETHODIMP
-imgRequestProxy::RequestDecode()
+bool
+imgRequestProxy::StartDecodingWithResult(uint32_t aFlags)
 {
   // Flag this, so we know to transfer the request if our owner changes
   mDecodeRequested = true;
 
   RefPtr<Image> image = GetImage();
   if (image) {
-    return image->RequestDecode();
+    return image->StartDecodingWithResult(aFlags);
   }
 
   if (GetOwner()) {
-    GetOwner()->RequestDecode();
+    GetOwner()->StartDecoding();
   }
 
-  return NS_OK;
+  return false;
 }
 
-
 NS_IMETHODIMP
 imgRequestProxy::LockImage()
 {
@@ -656,7 +645,7 @@ imgRequestProxy::PerformClone(imgINotificationObserver* aObserver,
 {
   NS_PRECONDITION(aClone, "Null out param");
 
-  LOG_SCOPE(GetImgLog(), "imgRequestProxy::Clone");
+  LOG_SCOPE(gImgLog, "imgRequestProxy::Clone");
 
   *aClone = nullptr;
   RefPtr<imgRequestProxy> clone = aAllocFn(this);
@@ -698,7 +687,8 @@ imgRequestProxy::GetImagePrincipal(nsIPrincipal** aPrincipal)
     return NS_ERROR_FAILURE;
   }
 
-  NS_ADDREF(*aPrincipal = GetOwner()->GetPrincipal());
+  nsCOMPtr<nsIPrincipal> principal = GetOwner()->GetPrincipal();
+  principal.forget(aPrincipal);
   return NS_OK;
 }
 
@@ -806,7 +796,7 @@ imgRequestProxy::Notify(int32_t aType, const mozilla::gfx::IntRect* aRect)
   MOZ_ASSERT(aType != imgINotificationObserver::LOAD_COMPLETE,
              "Should call OnLoadComplete");
 
-  LOG_FUNC_WITH_PARAM(GetImgLog(), "imgRequestProxy::Notify", "type",
+  LOG_FUNC_WITH_PARAM(gImgLog, "imgRequestProxy::Notify", "type",
                       NotificationTypeToString(aType));
 
   if (!mListener || mCanceled) {
@@ -816,16 +806,16 @@ imgRequestProxy::Notify(int32_t aType, const mozilla::gfx::IntRect* aRect)
   // Make sure the listener stays alive while we notify.
   nsCOMPtr<imgINotificationObserver> listener(mListener);
 
-  mListener->Notify(this, aType, aRect);
+  listener->Notify(this, aType, aRect);
 }
 
 void
 imgRequestProxy::OnLoadComplete(bool aLastPart)
 {
-  if (MOZ_LOG_TEST(GetImgLog(), LogLevel::Debug)) {
+  if (MOZ_LOG_TEST(gImgLog, LogLevel::Debug)) {
     nsAutoCString name;
     GetName(name);
-    LOG_FUNC_WITH_PARAM(GetImgLog(), "imgRequestProxy::OnLoadComplete",
+    LOG_FUNC_WITH_PARAM(gImgLog, "imgRequestProxy::OnLoadComplete",
                         "name", name.get());
   }
 
@@ -836,8 +826,8 @@ imgRequestProxy::OnLoadComplete(bool aLastPart)
 
   if (mListener && !mCanceled) {
     // Hold a ref to the listener while we call it, just in case.
-    nsCOMPtr<imgINotificationObserver> kungFuDeathGrip(mListener);
-    mListener->Notify(this, imgINotificationObserver::LOAD_COMPLETE, nullptr);
+    nsCOMPtr<imgINotificationObserver> listener(mListener);
+    listener->Notify(this, imgINotificationObserver::LOAD_COMPLETE, nullptr);
   }
 
   // If we're expecting more data from a multipart channel, re-add ourself
@@ -868,10 +858,10 @@ imgRequestProxy::OnLoadComplete(bool aLastPart)
 void
 imgRequestProxy::BlockOnload()
 {
-  if (MOZ_LOG_TEST(GetImgLog(), LogLevel::Debug)) {
+  if (MOZ_LOG_TEST(gImgLog, LogLevel::Debug)) {
     nsAutoCString name;
     GetName(name);
-    LOG_FUNC_WITH_PARAM(GetImgLog(), "imgRequestProxy::BlockOnload",
+    LOG_FUNC_WITH_PARAM(gImgLog, "imgRequestProxy::BlockOnload",
                         "name", name.get());
   }
 
@@ -884,10 +874,10 @@ imgRequestProxy::BlockOnload()
 void
 imgRequestProxy::UnblockOnload()
 {
-  if (MOZ_LOG_TEST(GetImgLog(), LogLevel::Debug)) {
+  if (MOZ_LOG_TEST(gImgLog, LogLevel::Debug)) {
     nsAutoCString name;
     GetName(name);
-    LOG_FUNC_WITH_PARAM(GetImgLog(), "imgRequestProxy::UnblockOnload",
+    LOG_FUNC_WITH_PARAM(gImgLog, "imgRequestProxy::UnblockOnload",
                         "name", name.get());
   }
 
@@ -1056,26 +1046,26 @@ class StaticBehaviour : public ProxyBehaviour
 public:
   explicit StaticBehaviour(mozilla::image::Image* aImage) : mImage(aImage) {}
 
-  virtual already_AddRefed<mozilla::image::Image>
+  already_AddRefed<mozilla::image::Image>
   GetImage() const override {
     RefPtr<mozilla::image::Image> image = mImage;
     return image.forget();
   }
 
-  virtual bool HasImage() const override {
+  bool HasImage() const override {
     return mImage;
   }
 
-  virtual already_AddRefed<ProgressTracker> GetProgressTracker()
+  already_AddRefed<ProgressTracker> GetProgressTracker()
     const override  {
     return mImage->GetProgressTracker();
   }
 
-  virtual imgRequest* GetOwner() const override {
+  imgRequest* GetOwner() const override {
     return nullptr;
   }
 
-  virtual void SetOwner(imgRequest* aOwner) override {
+  void SetOwner(imgRequest* aOwner) override {
     MOZ_ASSERT(!aOwner,
                "We shouldn't be giving static requests a non-null owner.");
   }
