@@ -7,6 +7,7 @@
 
 #include <cstdlib>
 
+#include "gfxPrefs.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsIDirectoryService.h"
 #include "nsIFile.h"
@@ -23,6 +24,7 @@ namespace image {
 using namespace gfx;
 
 using std::abs;
+using std::vector;
 
 ///////////////////////////////////////////////////////////////////////////////
 // General Helpers
@@ -42,9 +44,21 @@ using std::abs;
     return rv;                        \
   }
 
+#define ASSERT_GE_OR_RETURN(a, b, rv) \
+  EXPECT_GE(a, b);                    \
+  if (!((a) >= (b))) {                \
+    return rv;                        \
+  }
+
 #define ASSERT_LE_OR_RETURN(a, b, rv) \
   EXPECT_LE(a, b);                    \
-  if (!((a) <= (b))) {                 \
+  if (!((a) <= (b))) {                \
+    return rv;                        \
+  }
+
+#define ASSERT_LT_OR_RETURN(a, b, rv) \
+  EXPECT_LT(a, b);                    \
+  if (!((a) < (b))) {                 \
     return rv;                        \
   }
 
@@ -65,7 +79,7 @@ LoadFile(const char* aRelativePath)
 
   // Construct the final path by appending the working path to the current
   // working directory.
-  file->AppendNative(nsAutoCString(aRelativePath));
+  file->AppendNative(nsDependentCString(aRelativePath));
 
   // Construct an input stream for the requested file.
   nsCOMPtr<nsIInputStream> inputStream;
@@ -94,6 +108,13 @@ IsSolidColor(SourceSurface* aSurface,
                           aColor, aFuzz);
 }
 
+bool
+IsSolidPalettedColor(Decoder* aDecoder, uint8_t aColor)
+{
+  RawAccessFrameRef currentFrame = aDecoder->GetCurrentFrameRef();
+  return PalettedRectIsSolidColor(aDecoder, currentFrame->GetRect(), aColor);
+}
+
 bool
 RowsAreSolidColor(SourceSurface* aSurface,
                   int32_t aStartRow,
@@ -106,6 +127,18 @@ RowsAreSolidColor(SourceSurface* aSurface,
                           aColor, aFuzz);
 }
 
+bool
+PalettedRowsAreSolidColor(Decoder* aDecoder,
+                          int32_t aStartRow,
+                          int32_t aRowCount,
+                          uint8_t aColor)
+{
+  RawAccessFrameRef currentFrame = aDecoder->GetCurrentFrameRef();
+  IntRect frameRect = currentFrame->GetRect();
+  IntRect solidColorRect(frameRect.x, aStartRow, frameRect.width, aRowCount);
+  return PalettedRectIsSolidColor(aDecoder, solidColorRect, aColor);
+}
+
 bool
 RectIsSolidColor(SourceSurface* aSurface,
                  const IntRect& aRect,
@@ -149,6 +182,78 @@ RectIsSolidColor(SourceSurface* aSurface,
   return true;
 }
 
+bool
+PalettedRectIsSolidColor(Decoder* aDecoder, const IntRect& aRect, uint8_t aColor)
+{
+  RawAccessFrameRef currentFrame = aDecoder->GetCurrentFrameRef();
+  uint8_t* imageData;
+  uint32_t imageLength;
+  currentFrame->GetImageData(&imageData, &imageLength);
+  ASSERT_TRUE_OR_RETURN(imageData, false);
+
+  // Clamp to the frame rect. If any pixels outside the frame rect are included,
+  // we immediately fail, because such pixels don't have any "color" in the
+  // sense this function measures - they're transparent, and that doesn't
+  // necessarily correspond to any color palette index at all.
+  IntRect frameRect = currentFrame->GetRect();
+  ASSERT_EQ_OR_RETURN(imageLength, uint32_t(frameRect.Area()), false);
+  IntRect rect = aRect.Intersect(frameRect);
+  ASSERT_EQ_OR_RETURN(rect.Area(), aRect.Area(), false);
+
+  // Translate |rect| by |frameRect.TopLeft()| to reflect the fact that the
+  // frame rect's offset doesn't actually mean anything in terms of the
+  // in-memory representation of the surface. The image data starts at the upper
+  // left corner of the frame rect, in other words.
+  rect -= frameRect.TopLeft();
+
+  // Walk through the image data and make sure that the entire rect has the
+  // palette index |aColor|.
+  int32_t rowLength = frameRect.width;
+  for (int32_t row = rect.y; row < rect.YMost(); ++row) {
+    for (int32_t col = rect.x; col < rect.XMost(); ++col) {
+      int32_t i = row * rowLength + col;
+      ASSERT_EQ_OR_RETURN(aColor, imageData[i], false);
+    }
+  }
+
+  return true;
+}
+
+bool
+RowHasPixels(SourceSurface* aSurface,
+             int32_t aRow,
+             const vector<BGRAColor>& aPixels)
+{
+  ASSERT_GE_OR_RETURN(aRow, 0, false);
+
+  IntSize surfaceSize = aSurface->GetSize();
+  ASSERT_EQ_OR_RETURN(aPixels.size(), size_t(surfaceSize.width), false);
+  ASSERT_LT_OR_RETURN(aRow, surfaceSize.height, false);
+
+  RefPtr<DataSourceSurface> dataSurface = aSurface->GetDataSurface();
+  ASSERT_TRUE_OR_RETURN(dataSurface, false);
+
+  ASSERT_EQ_OR_RETURN(dataSurface->Stride(), surfaceSize.width * 4, false);
+
+  DataSourceSurface::ScopedMap mapping(dataSurface,
+                                       DataSourceSurface::MapType::READ);
+  ASSERT_TRUE_OR_RETURN(mapping.IsMapped(), false);
+
+  uint8_t* data = dataSurface->GetData();
+  ASSERT_TRUE_OR_RETURN(data != nullptr, false);
+
+  int32_t rowLength = dataSurface->Stride();
+  for (int32_t col = 0; col < surfaceSize.width; ++col) {
+    int32_t i = aRow * rowLength + col * 4;
+    ASSERT_EQ_OR_RETURN(aPixels[col].mBlue,  data[i + 0], false);
+    ASSERT_EQ_OR_RETURN(aPixels[col].mGreen, data[i + 1], false);
+    ASSERT_EQ_OR_RETURN(aPixels[col].mRed,   data[i + 2], false);
+    ASSERT_EQ_OR_RETURN(aPixels[col].mAlpha, data[i + 3], false);
+  }
+
+  return true;
+}
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // SurfacePipe Helpers
@@ -159,16 +264,17 @@ CreateTrivialDecoder()
 {
   gfxPrefs::GetSingleton();
   DecoderType decoderType = DecoderFactory::GetDecoderType("image/gif");
-  RefPtr<SourceBuffer> sourceBuffer = new SourceBuffer();
+  NotNull<RefPtr<SourceBuffer>> sourceBuffer = WrapNotNull(new SourceBuffer());
   RefPtr<Decoder> decoder =
-    DecoderFactory::CreateAnonymousDecoder(decoderType, sourceBuffer,
+    DecoderFactory::CreateAnonymousDecoder(decoderType, sourceBuffer, Nothing(),
                                            DefaultSurfaceFlags());
   return decoder.forget();
 }
 
-void AssertCorrectPipelineFinalState(SurfaceFilter* aFilter,
-                                     const gfx::IntRect& aInputSpaceRect,
-                                     const gfx::IntRect& aOutputSpaceRect)
+void
+AssertCorrectPipelineFinalState(SurfaceFilter* aFilter,
+                                const gfx::IntRect& aInputSpaceRect,
+                                const gfx::IntRect& aOutputSpaceRect)
 {
   EXPECT_TRUE(aFilter->IsSurfaceFinished());
   Maybe<SurfaceInvalidRect> invalidRect = aFilter->TakeInvalidRect();
@@ -183,7 +289,7 @@ CheckGeneratedImage(Decoder* aDecoder,
                     uint8_t aFuzz /* = 0 */)
 {
   RawAccessFrameRef currentFrame = aDecoder->GetCurrentFrameRef();
-  RefPtr<SourceSurface> surface = currentFrame->GetSurface();
+  RefPtr<SourceSurface> surface = currentFrame->GetSourceSurface();
   const IntSize surfaceSize = surface->GetSize();
 
   // This diagram shows how the surface is divided into regions that the code
@@ -224,15 +330,58 @@ CheckGeneratedImage(Decoder* aDecoder,
                                BGRAColor::Transparent(), aFuzz));
 }
 
-template <typename Func> void
-CheckSurfacePipeWrite(Decoder* aDecoder,
-                      SurfaceFilter* aFilter,
-                      Maybe<IntRect> aOutputRect,
-                      Maybe<IntRect> aInputRect,
-                      Maybe<IntRect> aInputWriteRect,
-                      Maybe<IntRect> aOutputWriteRect,
-                      uint8_t aFuzz,
-                      Func aFunc)
+void
+CheckGeneratedPalettedImage(Decoder* aDecoder, const IntRect& aRect)
+{
+  RawAccessFrameRef currentFrame = aDecoder->GetCurrentFrameRef();
+  IntSize imageSize = currentFrame->GetImageSize();
+
+  // This diagram shows how the surface is divided into regions that the code
+  // below tests for the correct content. The output rect is the bounds of the
+  // region labeled 'C'.
+  //
+  // +---------------------------+
+  // |             A             |
+  // +---------+--------+--------+
+  // |    B    |   C    |   D    |
+  // +---------+--------+--------+
+  // |             E             |
+  // +---------------------------+
+
+  // Check that the output rect itself is all 255's. (Region 'C'.)
+  EXPECT_TRUE(PalettedRectIsSolidColor(aDecoder, aRect, 255));
+
+  // Check that the area above the output rect is all 0's. (Region 'A'.)
+  EXPECT_TRUE(PalettedRectIsSolidColor(aDecoder,
+                                       IntRect(0, 0, imageSize.width, aRect.y),
+                                       0));
+
+  // Check that the area to the left of the output rect is all 0's. (Region 'B'.)
+  EXPECT_TRUE(PalettedRectIsSolidColor(aDecoder,
+                                       IntRect(0, aRect.y, aRect.x, aRect.YMost()),
+                                       0));
+
+  // Check that the area to the right of the output rect is all 0's. (Region 'D'.)
+  const int32_t widthOnRight = imageSize.width - aRect.XMost();
+  EXPECT_TRUE(PalettedRectIsSolidColor(aDecoder,
+                                       IntRect(aRect.XMost(), aRect.y, widthOnRight, aRect.YMost()),
+                                       0));
+
+  // Check that the area below the output rect is transparent. (Region 'E'.)
+  const int32_t heightBelow = imageSize.height - aRect.YMost();
+  EXPECT_TRUE(PalettedRectIsSolidColor(aDecoder,
+                                       IntRect(0, aRect.YMost(), imageSize.width, heightBelow),
+                                       0));
+}
+
+void
+CheckWritePixels(Decoder* aDecoder,
+                 SurfaceFilter* aFilter,
+                 const Maybe<IntRect>& aOutputRect /* = Nothing() */,
+                 const Maybe<IntRect>& aInputRect /* = Nothing() */,
+                 const Maybe<IntRect>& aInputWriteRect /* = Nothing() */,
+                 const Maybe<IntRect>& aOutputWriteRect /* = Nothing() */,
+                 uint8_t aFuzz /* = 0 */)
 {
   IntRect outputRect = aOutputRect.valueOr(IntRect(0, 0, 100, 100));
   IntRect inputRect = aInputRect.valueOr(IntRect(0, 0, 100, 100));
@@ -241,7 +390,10 @@ CheckSurfacePipeWrite(Decoder* aDecoder,
 
   // Fill the image.
   int32_t count = 0;
-  auto result = aFunc(count);
+  auto result = aFilter->WritePixels<uint32_t>([&] {
+    ++count;
+    return AsVariant(BGRAColor::Green().AsPixel());
+  });
   EXPECT_EQ(WriteState::FINISHED, result);
   EXPECT_EQ(inputWriteRect.width * inputWriteRect.height, count);
 
@@ -249,7 +401,10 @@ CheckSurfacePipeWrite(Decoder* aDecoder,
 
   // Attempt to write more data and make sure nothing changes.
   const int32_t oldCount = count;
-  result = aFunc(count);
+  result = aFilter->WritePixels<uint32_t>([&] {
+    ++count;
+    return AsVariant(BGRAColor::Green().AsPixel());
+  });
   EXPECT_EQ(oldCount, count);
   EXPECT_EQ(WriteState::FINISHED, result);
   EXPECT_TRUE(aFilter->IsSurfaceFinished());
@@ -267,58 +422,13 @@ CheckSurfacePipeWrite(Decoder* aDecoder,
 }
 
 void
-CheckWritePixels(Decoder* aDecoder,
-                 SurfaceFilter* aFilter,
-                 Maybe<IntRect> aOutputRect /* = Nothing() */,
-                 Maybe<IntRect> aInputRect /* = Nothing() */,
-                 Maybe<IntRect> aInputWriteRect /* = Nothing() */,
-                 Maybe<IntRect> aOutputWriteRect /* = Nothing() */,
-                 uint8_t aFuzz /* = 0 */)
-{
-  CheckSurfacePipeWrite(aDecoder, aFilter,
-                        aOutputRect, aInputRect,
-                        aInputWriteRect, aOutputWriteRect,
-                        aFuzz,
-                        [&](int32_t& aCount) {
-    return aFilter->WritePixels<uint32_t>([&] {
-      ++aCount;
-      return AsVariant(BGRAColor::Green().AsPixel());
-    });
-  });
-}
-
-void
-CheckWriteRows(Decoder* aDecoder,
-               SurfaceFilter* aFilter,
-               Maybe<IntRect> aOutputRect /* = Nothing() */,
-               Maybe<IntRect> aInputRect /* = Nothing() */,
-               Maybe<IntRect> aInputWriteRect /* = Nothing() */,
-               Maybe<IntRect> aOutputWriteRect /* = Nothing() */,
-               uint8_t aFuzz /* = 0 */)
-{
-  CheckSurfacePipeWrite(aDecoder, aFilter,
-                        aOutputRect, aInputRect,
-                        aInputWriteRect, aOutputWriteRect,
-                        aFuzz,
-                        [&](int32_t& aCount) {
-    return aFilter->WriteRows<uint32_t>([&](uint32_t* aRow, uint32_t aLength) {
-      for (; aLength > 0; --aLength, ++aRow, ++aCount) {
-        *aRow = BGRAColor::Green().AsPixel();
-      }
-      return Nothing();
-    });
-  });
-}
-
-template <typename Func> void
-CheckPalettedSurfacePipeWrite(Decoder* aDecoder,
-                              SurfaceFilter* aFilter,
-                              Maybe<IntRect> aOutputRect,
-                              Maybe<IntRect> aInputRect,
-                              Maybe<IntRect> aInputWriteRect,
-                              Maybe<IntRect> aOutputWriteRect,
-                              uint8_t aFuzz,
-                              Func aFunc)
+CheckPalettedWritePixels(Decoder* aDecoder,
+                         SurfaceFilter* aFilter,
+                         const Maybe<IntRect>& aOutputRect /* = Nothing() */,
+                         const Maybe<IntRect>& aInputRect /* = Nothing() */,
+                         const Maybe<IntRect>& aInputWriteRect /* = Nothing() */,
+                         const Maybe<IntRect>& aOutputWriteRect /* = Nothing() */,
+                         uint8_t aFuzz /* = 0 */)
 {
   IntRect outputRect = aOutputRect.valueOr(IntRect(0, 0, 100, 100));
   IntRect inputRect = aInputRect.valueOr(IntRect(0, 0, 100, 100));
@@ -327,7 +437,10 @@ CheckPalettedSurfacePipeWrite(Decoder* aDecoder,
 
   // Fill the image.
   int32_t count = 0;
-  auto result = aFunc(count);
+  auto result = aFilter->WritePixels<uint8_t>([&] {
+    ++count;
+    return AsVariant(uint8_t(255));
+  });
   EXPECT_EQ(WriteState::FINISHED, result);
   EXPECT_EQ(inputWriteRect.width * inputWriteRect.height, count);
 
@@ -335,7 +448,10 @@ CheckPalettedSurfacePipeWrite(Decoder* aDecoder,
 
   // Attempt to write more data and make sure nothing changes.
   const int32_t oldCount = count;
-  result = aFunc(count);
+  result = aFilter->WritePixels<uint8_t>([&] {
+    ++count;
+    return AsVariant(uint8_t(255));
+  });
   EXPECT_EQ(oldCount, count);
   EXPECT_EQ(WriteState::FINISHED, result);
   EXPECT_TRUE(aFilter->IsSurfaceFinished());
@@ -360,50 +476,6 @@ CheckPalettedSurfacePipeWrite(Decoder* aDecoder,
   }
 }
 
-void
-CheckPalettedWritePixels(Decoder* aDecoder,
-                         SurfaceFilter* aFilter,
-                         Maybe<IntRect> aOutputRect /* = Nothing() */,
-                         Maybe<IntRect> aInputRect /* = Nothing() */,
-                         Maybe<IntRect> aInputWriteRect /* = Nothing() */,
-                         Maybe<IntRect> aOutputWriteRect /* = Nothing() */,
-                         uint8_t aFuzz /* = 0 */)
-{
-  CheckPalettedSurfacePipeWrite(aDecoder, aFilter,
-                                aOutputRect, aInputRect,
-                                aInputWriteRect, aOutputWriteRect,
-                                aFuzz,
-                                [&](int32_t& aCount) {
-    return aFilter->WritePixels<uint8_t>([&] {
-      ++aCount;
-      return AsVariant(uint8_t(255));
-    });
-  });
-}
-
-void
-CheckPalettedWriteRows(Decoder* aDecoder,
-                       SurfaceFilter* aFilter,
-                       Maybe<IntRect> aOutputRect /* = Nothing() */,
-                       Maybe<IntRect> aInputRect /* = Nothing() */,
-                       Maybe<IntRect> aInputWriteRect /* = Nothing() */,
-                       Maybe<IntRect> aOutputWriteRect /* = Nothing() */,
-                       uint8_t aFuzz /* = 0*/)
-{
-  CheckPalettedSurfacePipeWrite(aDecoder, aFilter,
-                                aOutputRect, aInputRect,
-                                aInputWriteRect, aOutputWriteRect,
-                                aFuzz,
-                                [&](int32_t& aCount) {
-    return aFilter->WriteRows<uint8_t>([&](uint8_t* aRow, uint32_t aLength) {
-      for (; aLength > 0; --aLength, ++aRow, ++aCount) {
-        *aRow = uint8_t(255);
-      }
-      return Nothing();
-    });
-  });
-}
-
 
 ///////////////////////////////////////////////////////////////////////////////
 // Test Data
@@ -462,6 +534,40 @@ ImageTestCase CorruptTestCase()
                        TEST_CASE_HAS_ERROR);
 }
 
+ImageTestCase CorruptBMPWithTruncatedHeader()
+{
+  // This BMP has a header which is truncated right between the BIH and the
+  // bitfields, which is a particularly error-prone place w.r.t. the BMP decoder
+  // state machine.
+  return ImageTestCase("invalid-truncated-metadata.bmp", "image/bmp",
+                       IntSize(100, 100), TEST_CASE_HAS_ERROR);
+}
+
+ImageTestCase CorruptICOWithBadBMPWidthTestCase()
+{
+  // This ICO contains a BMP icon which has a width that doesn't match the size
+  // listed in the corresponding ICO directory entry.
+  return ImageTestCase("corrupt-with-bad-bmp-width.ico", "image/x-icon",
+                       IntSize(100, 100), TEST_CASE_HAS_ERROR);
+}
+
+ImageTestCase CorruptICOWithBadBMPHeightTestCase()
+{
+  // This ICO contains a BMP icon which has a height that doesn't match the size
+  // listed in the corresponding ICO directory entry.
+  return ImageTestCase("corrupt-with-bad-bmp-height.ico", "image/x-icon",
+                       IntSize(100, 100), TEST_CASE_HAS_ERROR);
+}
+
+ImageTestCase CorruptICOWithBadBppTestCase()
+{
+  // This test case is an ICO with a BPP (15) in the ICO header which differs
+  // from that in the BMP header itself (1). It should ignore the ICO BPP when
+  // the BMP BPP is available and thus correctly decode the image.
+  return ImageTestCase("corrupt-with-bad-ico-bpp.ico", "image/x-icon",
+                       IntSize(100, 100), TEST_CASE_IS_TRANSPARENT);
+}
+
 ImageTestCase TransparentPNGTestCase()
 {
   return ImageTestCase("transparent.png", "image/png", IntSize(32, 32),
@@ -510,5 +616,68 @@ ImageTestCase NoFrameDelayGIFTestCase()
   return ImageTestCase("no-frame-delay.gif", "image/gif", IntSize(100, 100));
 }
 
+ImageTestCase ExtraImageSubBlocksAnimatedGIFTestCase()
+{
+  // This is a corrupt GIF that has extra image sub blocks between the first and
+  // second frame.
+  return ImageTestCase("animated-with-extra-image-sub-blocks.gif", "image/gif",
+                       IntSize(100, 100));
+}
+
+ImageTestCase DownscaledPNGTestCase()
+{
+  // This testcase (and all the other "downscaled") testcases) consists of 25
+  // lines of green, followed by 25 lines of red, followed by 25 lines of green,
+  // followed by 25 more lines of red. It's intended that tests downscale it
+  // from 100x100 to 20x20, so we specify a 20x20 output size.
+  return ImageTestCase("downscaled.png", "image/png", IntSize(100, 100),
+                       IntSize(20, 20));
+}
+
+ImageTestCase DownscaledGIFTestCase()
+{
+  return ImageTestCase("downscaled.gif", "image/gif", IntSize(100, 100),
+                       IntSize(20, 20));
+}
+
+ImageTestCase DownscaledJPGTestCase()
+{
+  return ImageTestCase("downscaled.jpg", "image/jpeg", IntSize(100, 100),
+                       IntSize(20, 20));
+}
+
+ImageTestCase DownscaledBMPTestCase()
+{
+  return ImageTestCase("downscaled.bmp", "image/bmp", IntSize(100, 100),
+                       IntSize(20, 20));
+}
+
+ImageTestCase DownscaledICOTestCase()
+{
+  return ImageTestCase("downscaled.ico", "image/x-icon", IntSize(100, 100),
+                       IntSize(20, 20), TEST_CASE_IS_TRANSPARENT);
+}
+
+ImageTestCase DownscaledIconTestCase()
+{
+  return ImageTestCase("downscaled.icon", "image/icon", IntSize(100, 100),
+                       IntSize(20, 20), TEST_CASE_IS_TRANSPARENT);
+}
+
+ImageTestCase DownscaledTransparentICOWithANDMaskTestCase()
+{
+  // This test case is an ICO with AND mask transparency. We want to ensure that
+  // we can downscale it without crashing or triggering ASAN failures, but its
+  // content isn't simple to verify, so for now we don't check the output.
+  return ImageTestCase("transparent-ico-with-and-mask.ico", "image/x-icon",
+                       IntSize(32, 32), IntSize(20, 20),
+                       TEST_CASE_IS_TRANSPARENT | TEST_CASE_IGNORE_OUTPUT);
+}
+
+ImageTestCase TruncatedSmallGIFTestCase()
+{
+  return ImageTestCase("green-1x1-truncated.gif", "image/gif", IntSize(1, 1));
+}
+
 } // namespace image
 } // namespace mozilla
