@@ -1,6 +1,7 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
+Cu.import("resource://services-sync/browserid_identity.js");
 Cu.import("resource://services-sync/constants.js");
 Cu.import("resource://services-sync/engines.js");
 Cu.import("resource://services-sync/engines/clients.js");
@@ -29,6 +30,11 @@ Service.engineManager.register(CatapultEngine);
 var scheduler = new SyncScheduler(Service);
 var clientsEngine = Service.clientsEngine;
 
+// Don't remove stale clients when syncing. This is a test-only workaround
+// that lets us add clients directly to the store, without losing them on
+// the next sync.
+clientsEngine._removeRemoteClient = id => {};
+
 function sync_httpd_setup() {
   let global = new ServerWBO("global", {
     syncID: Service.syncID,
@@ -43,40 +49,31 @@ function sync_httpd_setup() {
   let upd = collectionsHelper.with_updated_collection;
 
   return httpd_setup({
-    "/1.1/johndoe/storage/meta/global": upd("meta", global.handler()),
-    "/1.1/johndoe/info/collections": collectionsHelper.handler,
-    "/1.1/johndoe/storage/crypto/keys":
+    "/1.1/johndoe@mozilla.com/storage/meta/global": upd("meta", global.handler()),
+    "/1.1/johndoe@mozilla.com/info/collections": collectionsHelper.handler,
+    "/1.1/johndoe@mozilla.com/storage/crypto/keys":
       upd("crypto", (new ServerWBO("keys")).handler()),
-    "/1.1/johndoe/storage/clients": upd("clients", clientsColl.handler()),
-    "/user/1.0/johndoe/node/weave": httpd_handler(200, "OK", "null")
+    "/1.1/johndoe@mozilla.com/storage/clients": upd("clients", clientsColl.handler())
   });
 }
 
-function setUp(server) {
-  let deferred = Promise.defer();
-  configureIdentity({username: "johndoe"}).then(() => {
-    Service.clusterURL = server.baseURI + "/";
+async function setUp(server) {
+  await configureIdentity({username: "johndoe@mozilla.com"}, server);
 
-    generateNewKeys(Service.collectionKeys);
-    let serverKeys = Service.collectionKeys.asWBO("crypto", "keys");
-    serverKeys.encrypt(Service.identity.syncKeyBundle);
-    let result = serverKeys.upload(Service.resource(Service.cryptoKeysURL)).success;
-    deferred.resolve(result);
-  });
-  return deferred.promise;
+  generateNewKeys(Service.collectionKeys);
+  let serverKeys = Service.collectionKeys.asWBO("crypto", "keys");
+  serverKeys.encrypt(Service.identity.syncKeyBundle);
+  let result = serverKeys.upload(Service.resource(Service.cryptoKeysURL)).success;
+  return result;
 }
 
-function cleanUpAndGo(server) {
-  let deferred = Promise.defer();
-  Utils.nextTick(function () {
-    Service.startOver();
-    if (server) {
-      server.stop(deferred.resolve);
-    } else {
-      deferred.resolve();
-    }
-  });
-  return deferred.promise;
+async function cleanUpAndGo(server) {
+  await promiseNextTick();
+  clientsEngine._store.wipe();
+  Service.startOver();
+  if (server) {
+    await promiseStopServer(server);
+  }
 }
 
 function run_test() {
@@ -84,12 +81,8 @@ function run_test() {
 
   Log.repository.getLogger("Sync.Service").level = Log.Level.Trace;
   Log.repository.getLogger("Sync.scheduler").level = Log.Level.Trace;
+  validate_all_future_pings();
 
-  // The scheduler checks Weave.fxaEnabled to determine whether to use
-  // FxA defaults or legacy defaults.  As .fxaEnabled checks the username, we
-  // set a username here then reset the default to ensure they are used.
-  ensureLegacyIdentityManager();
-  setBasicCredentials("johndoe");
   scheduler.setDefaults();
 
   run_next_test();
@@ -109,30 +102,30 @@ add_test(function test_prefAttributes() {
   do_check_eq(scheduler.nextSync, Math.floor(TIMESTAMP1 / 1000) * 1000);
 
   _("'syncInterval' defaults to singleDeviceInterval.");
-  do_check_eq(Svc.Prefs.get('syncInterval'), undefined);
+  do_check_eq(Svc.Prefs.get("syncInterval"), undefined);
   do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
 
   _("'syncInterval' corresponds to a preference setting.");
   scheduler.syncInterval = INTERVAL;
   do_check_eq(scheduler.syncInterval, INTERVAL);
-  do_check_eq(Svc.Prefs.get('syncInterval'), INTERVAL);
+  do_check_eq(Svc.Prefs.get("syncInterval"), INTERVAL);
 
   _("'syncThreshold' corresponds to preference, defaults to SINGLE_USER_THRESHOLD");
-  do_check_eq(Svc.Prefs.get('syncThreshold'), undefined);
+  do_check_eq(Svc.Prefs.get("syncThreshold"), undefined);
   do_check_eq(scheduler.syncThreshold, SINGLE_USER_THRESHOLD);
   scheduler.syncThreshold = THRESHOLD;
   do_check_eq(scheduler.syncThreshold, THRESHOLD);
 
   _("'globalScore' corresponds to preference, defaults to zero.");
-  do_check_eq(Svc.Prefs.get('globalScore'), 0);
+  do_check_eq(Svc.Prefs.get("globalScore"), 0);
   do_check_eq(scheduler.globalScore, 0);
   scheduler.globalScore = SCORE;
   do_check_eq(scheduler.globalScore, SCORE);
-  do_check_eq(Svc.Prefs.get('globalScore'), SCORE);
+  do_check_eq(Svc.Prefs.get("globalScore"), SCORE);
 
   _("Intervals correspond to default preferences.");
   do_check_eq(scheduler.singleDeviceInterval,
-              Svc.Prefs.get("scheduler.sync11.singleDeviceInterval") * 1000);
+              Svc.Prefs.get("scheduler.fxa.singleDeviceInterval") * 1000);
   do_check_eq(scheduler.idleInterval,
               Svc.Prefs.get("scheduler.idleInterval") * 1000);
   do_check_eq(scheduler.activeInterval,
@@ -141,30 +134,42 @@ add_test(function test_prefAttributes() {
               Svc.Prefs.get("scheduler.immediateInterval") * 1000);
 
   _("Custom values for prefs will take effect after a restart.");
-  Svc.Prefs.set("scheduler.sync11.singleDeviceInterval", 42);
-  Svc.Prefs.set("scheduler.idleInterval", 23);
-  Svc.Prefs.set("scheduler.activeInterval", 18);
+  Svc.Prefs.set("scheduler.fxa.singleDeviceInterval", 420);
+  Svc.Prefs.set("scheduler.idleInterval", 230);
+  Svc.Prefs.set("scheduler.activeInterval", 180);
   Svc.Prefs.set("scheduler.immediateInterval", 31415);
   scheduler.setDefaults();
-  do_check_eq(scheduler.idleInterval, 23000);
-  do_check_eq(scheduler.singleDeviceInterval, 42000);
-  do_check_eq(scheduler.activeInterval, 18000);
+  do_check_eq(scheduler.idleInterval, 230000);
+  do_check_eq(scheduler.singleDeviceInterval, 420000);
+  do_check_eq(scheduler.activeInterval, 180000);
   do_check_eq(scheduler.immediateInterval, 31415000);
 
+  _("Custom values for interval prefs can't be less than 60 seconds.");
+  Svc.Prefs.set("scheduler.fxa.singleDeviceInterval", 42);
+  Svc.Prefs.set("scheduler.idleInterval", 50);
+  Svc.Prefs.set("scheduler.activeInterval", 50);
+  Svc.Prefs.set("scheduler.immediateInterval", 10);
+  scheduler.setDefaults();
+  do_check_eq(scheduler.idleInterval, 60000);
+  do_check_eq(scheduler.singleDeviceInterval, 60000);
+  do_check_eq(scheduler.activeInterval, 60000);
+  do_check_eq(scheduler.immediateInterval, 60000);
+
   Svc.Prefs.resetBranch("");
   scheduler.setDefaults();
   run_next_test();
 });
 
-add_identity_test(this, function* test_updateClientMode() {
+add_task(async function test_updateClientMode() {
   _("Test updateClientMode adjusts scheduling attributes based on # of clients appropriately");
   do_check_eq(scheduler.syncThreshold, SINGLE_USER_THRESHOLD);
   do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
   do_check_false(scheduler.numClients > 1);
   do_check_false(scheduler.idle);
 
-  // Trigger a change in interval & threshold by adding a client.
-  clientsEngine._store.create({id: "foo", cleartext: "bar"});
+  // Trigger a change in interval & threshold by noting there are multiple clients.
+  Svc.Prefs.set("clients.devices.desktop", 1);
+  Svc.Prefs.set("clients.devices.mobile", 1);
   scheduler.updateClientMode();
 
   do_check_eq(scheduler.syncThreshold, MULTI_DEVICE_THRESHOLD);
@@ -174,6 +179,7 @@ add_identity_test(this, function* test_updateClientMode() {
 
   // Resets the number of clients to 0.
   clientsEngine.resetClient();
+  Svc.Prefs.reset("clients.devices.mobile");
   scheduler.updateClientMode();
 
   // Goes back to single user if # clients is 1.
@@ -183,10 +189,12 @@ add_identity_test(this, function* test_updateClientMode() {
   do_check_false(scheduler.numClients > 1);
   do_check_false(scheduler.idle);
 
-  yield cleanUpAndGo();
+  await cleanUpAndGo();
 });
 
-add_identity_test(this, function* test_masterpassword_locked_retry_interval() {
+add_task(async function test_masterpassword_locked_retry_interval() {
+  enableValidationPrefs();
+
   _("Test Status.login = MASTER_PASSWORD_LOCKED results in reschedule at MASTER_PASSWORD interval");
   let loginFailed = false;
   Svc.Obs.add("weave:service:login:error", function onLoginError() {
@@ -197,19 +205,19 @@ add_identity_test(this, function* test_masterpassword_locked_retry_interval() {
   let rescheduleInterval = false;
 
   let oldScheduleAtInterval = SyncScheduler.prototype.scheduleAtInterval;
-  SyncScheduler.prototype.scheduleAtInterval = function (interval) {
+  SyncScheduler.prototype.scheduleAtInterval = function(interval) {
     rescheduleInterval = true;
     do_check_eq(interval, MASTER_PASSWORD_LOCKED_RETRY_INTERVAL);
   };
 
   let oldVerifyLogin = Service.verifyLogin;
-  Service.verifyLogin = function () {
+  Service.verifyLogin = function() {
     Status.login = MASTER_PASSWORD_LOCKED;
     return false;
   };
 
   let server = sync_httpd_setup();
-  yield setUp(server);
+  await setUp(server);
 
   Service.sync();
 
@@ -220,10 +228,10 @@ add_identity_test(this, function* test_masterpassword_locked_retry_interval() {
   Service.verifyLogin = oldVerifyLogin;
   SyncScheduler.prototype.scheduleAtInterval = oldScheduleAtInterval;
 
-  yield cleanUpAndGo(server);
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_calculateBackoff() {
+add_task(async function test_calculateBackoff() {
   do_check_eq(Status.backoffInterval, 0);
 
   // Test no interval larger than the maximum backoff is used if
@@ -242,25 +250,26 @@ add_identity_test(this, function* test_calculateBackoff() {
 
   do_check_eq(backoffInterval, MAXIMUM_BACKOFF_INTERVAL + 10);
 
-  yield cleanUpAndGo();
+  await cleanUpAndGo();
 });
 
-add_identity_test(this, function* test_scheduleNextSync_nowOrPast() {
-  let deferred = Promise.defer();
-  Svc.Obs.add("weave:service:sync:finish", function onSyncFinish() {
-    Svc.Obs.remove("weave:service:sync:finish", onSyncFinish);
-    cleanUpAndGo(server).then(deferred.resolve);
-  });
+add_task(async function test_scheduleNextSync_nowOrPast() {
+  enableValidationPrefs();
+
+  let promiseObserved = promiseOneObserver("weave:service:sync:finish");
 
   let server = sync_httpd_setup();
-  yield setUp(server);
+  await setUp(server);
 
   // We're late for a sync...
   scheduler.scheduleNextSync(-1);
-  yield deferred.promise;
+  await promiseObserved;
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_scheduleNextSync_future_noBackoff() {
+add_task(async function test_scheduleNextSync_future_noBackoff() {
+  enableValidationPrefs();
+
   _("scheduleNextSync() uses the current syncInterval if no interval is provided.");
   // Test backoffInterval is 0 as expected.
   do_check_eq(Status.backoffInterval, 0);
@@ -306,10 +315,12 @@ add_identity_test(this, function* test_scheduleNextSync_future_noBackoff() {
   do_check_true(scheduler.nextSync <= Date.now() + 1);
   do_check_eq(scheduler.syncTimer.delay, 1);
 
-  yield cleanUpAndGo();
+  await cleanUpAndGo();
 });
 
-add_identity_test(this, function* test_scheduleNextSync_future_backoff() {
+add_task(async function test_scheduleNextSync_future_backoff() {
+  enableValidationPrefs();
+
  _("scheduleNextSync() will honour backoff in all scheduling requests.");
   // Let's take a backoff interval that's bigger than the default sync interval.
   const BACKOFF = 7337;
@@ -356,12 +367,14 @@ add_identity_test(this, function* test_scheduleNextSync_future_backoff() {
   do_check_true(scheduler.nextSync <= Date.now() + Status.backoffInterval);
   do_check_eq(scheduler.syncTimer.delay, Status.backoffInterval);
 
-  yield cleanUpAndGo();
+  await cleanUpAndGo();
 });
 
-add_identity_test(this, function* test_handleSyncError() {
+add_task(async function test_handleSyncError() {
+  enableValidationPrefs();
+
   let server = sync_httpd_setup();
-  yield setUp(server);
+  await setUp(server);
 
   // Force sync to fail.
   Svc.Prefs.set("firstSync", "notReady");
@@ -415,19 +428,18 @@ add_identity_test(this, function* test_handleSyncError() {
   scheduler.syncTimer.clear();
 
   _("Arrange for a successful sync to reset the scheduler error count");
-  let deferred = Promise.defer();
-  Svc.Obs.add("weave:service:sync:finish", function onSyncFinish() {
-    Svc.Obs.remove("weave:service:sync:finish", onSyncFinish);
-    cleanUpAndGo(server).then(deferred.resolve);
-  });
+  let promiseObserved = promiseOneObserver("weave:service:sync:finish");
   Svc.Prefs.set("firstSync", "wipeRemote");
   scheduler.scheduleNextSync(-1);
-  yield deferred.promise;
+  await promiseObserved;
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_client_sync_finish_updateClientMode() {
+add_task(async function test_client_sync_finish_updateClientMode() {
+  enableValidationPrefs();
+
   let server = sync_httpd_setup();
-  yield setUp(server);
+  await setUp(server);
 
   // Confirm defaults.
   do_check_eq(scheduler.syncThreshold, SINGLE_USER_THRESHOLD);
@@ -435,7 +447,9 @@ add_identity_test(this, function* test_client_sync_finish_updateClientMode() {
   do_check_false(scheduler.idle);
 
   // Trigger a change in interval & threshold by adding a client.
-  clientsEngine._store.create({id: "foo", cleartext: "bar"});
+  clientsEngine._store.create(
+    { id: "foo", cleartext: { os: "mobile", version: "0.01", type: "desktop" } }
+  );
   do_check_false(scheduler.numClients > 1);
   scheduler.updateClientMode();
   Service.sync();
@@ -447,6 +461,9 @@ add_identity_test(this, function* test_client_sync_finish_updateClientMode() {
 
   // Resets the number of clients to 0.
   clientsEngine.resetClient();
+  // Also re-init the server, or we suck our "foo" client back down.
+  await setUp(server);
+
   Service.sync();
 
   // Goes back to single user if # clients is 1.
@@ -456,27 +473,26 @@ add_identity_test(this, function* test_client_sync_finish_updateClientMode() {
   do_check_false(scheduler.numClients > 1);
   do_check_false(scheduler.idle);
 
-  yield cleanUpAndGo(server);
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_autoconnect_nextSync_past() {
-  let deferred = Promise.defer();
+add_task(async function test_autoconnect_nextSync_past() {
+  enableValidationPrefs();
+
+  let promiseObserved = promiseOneObserver("weave:service:sync:finish");
   // nextSync will be 0 by default, so it's way in the past.
 
-  Svc.Obs.add("weave:service:sync:finish", function onSyncFinish() {
-    Svc.Obs.remove("weave:service:sync:finish", onSyncFinish);
-    cleanUpAndGo(server).then(deferred.resolve);
-  });
-
   let server = sync_httpd_setup();
-  yield setUp(server);
+  await setUp(server);
 
   scheduler.delayedAutoConnect(0);
-  yield deferred.promise;
+  await promiseObserved;
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_autoconnect_nextSync_future() {
-  let deferred = Promise.defer();
+add_task(async function test_autoconnect_nextSync_future() {
+  enableValidationPrefs();
+
   let previousSync = Date.now() + scheduler.syncInterval / 2;
   scheduler.nextSync = previousSync;
   // nextSync rounds to the nearest second.
@@ -489,61 +505,55 @@ add_identity_test(this, function* test_autoconnect_nextSync_future() {
   }
   Svc.Obs.add("weave:service:login:start", onLoginStart);
 
-  waitForZeroTimer(function () {
-    do_check_eq(scheduler.nextSync, expectedSync);
-    do_check_true(scheduler.syncTimer.delay >= expectedInterval);
-
-    Svc.Obs.remove("weave:service:login:start", onLoginStart);
-    cleanUpAndGo().then(deferred.resolve);
-  });
-
-  yield configureIdentity({username: "johndoe"});
+  await configureIdentity({username: "johndoe@mozilla.com"});
   scheduler.delayedAutoConnect(0);
-  yield deferred.promise;
+  await promiseZeroTimer();
+
+  do_check_eq(scheduler.nextSync, expectedSync);
+  do_check_true(scheduler.syncTimer.delay >= expectedInterval);
+
+  Svc.Obs.remove("weave:service:login:start", onLoginStart);
+  await cleanUpAndGo();
 });
 
-// XXX - this test can't be run with the browserid identity as it relies
-// on the syncKey getter behaving in a certain way...
-add_task(function* test_autoconnect_mp_locked() {
+add_task(async function test_autoconnect_mp_locked() {
   let server = sync_httpd_setup();
-  yield setUp(server);
+  await setUp(server);
 
   // Pretend user did not unlock master password.
   let origLocked = Utils.mpLocked;
   Utils.mpLocked = () => true;
 
-  let origGetter = Service.identity.__lookupGetter__("syncKey");
-  let origSetter = Service.identity.__lookupSetter__("syncKey");
-  delete Service.identity.syncKey;
-  Service.identity.__defineGetter__("syncKey", function() {
-    _("Faking Master Password entry cancelation.");
-    throw "User canceled Master Password entry";
-  });
 
-  let deferred = Promise.defer();
+  let origEnsureMPUnlocked = Utils.ensureMPUnlocked;
+  Utils.ensureMPUnlocked = () => {
+    _("Faking Master Password entry cancelation.");
+    return false;
+  }
+  let origCanFetchKeys = Service.identity._canFetchKeys;
+  Service.identity._canFetchKeys = () => false;
+
   // A locked master password will still trigger a sync, but then we'll hit
   // MASTER_PASSWORD_LOCKED and hence MASTER_PASSWORD_LOCKED_RETRY_INTERVAL.
-  Svc.Obs.add("weave:service:login:error", function onLoginError() {
-    Svc.Obs.remove("weave:service:login:error", onLoginError);
-    Utils.nextTick(function aLittleBitAfterLoginError() {
-      do_check_eq(Status.login, MASTER_PASSWORD_LOCKED);
-
-      Utils.mpLocked = origLocked;
-      delete Service.identity.syncKey;
-      Service.identity.__defineGetter__("syncKey", origGetter);
-      Service.identity.__defineSetter__("syncKey", origSetter);
-
-      cleanUpAndGo(server).then(deferred.resolve);
-    });
-  });
+  let promiseObserved = promiseOneObserver("weave:service:login:error");
 
   scheduler.delayedAutoConnect(0);
-  yield deferred.promise;
+  await promiseObserved;
+
+  await promiseNextTick();
+
+  do_check_eq(Status.login, MASTER_PASSWORD_LOCKED);
+
+  Utils.mpLocked = origLocked;
+  Utils.ensureMPUnlocked = origEnsureMPUnlocked;
+  Service.identity._canFetchKeys = origCanFetchKeys;
+
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_no_autoconnect_during_wizard() {
+add_task(async function test_no_autoconnect_during_wizard() {
   let server = sync_httpd_setup();
-  yield setUp(server);
+  await setUp(server);
 
   // Simulate the Sync setup wizard.
   Svc.Prefs.set("firstSync", "notReady");
@@ -554,18 +564,15 @@ add_identity_test(this, function* test_no_autoconnect_during_wizard() {
   }
   Svc.Obs.add("weave:service:login:start", onLoginStart);
 
-  let deferred = Promise.defer();
-  waitForZeroTimer(function () {
-    Svc.Obs.remove("weave:service:login:start", onLoginStart);
-    cleanUpAndGo(server).then(deferred.resolve);
-  });
-
   scheduler.delayedAutoConnect(0);
-  yield deferred.promise;
+  await promiseZeroTimer();
+  Svc.Obs.remove("weave:service:login:start", onLoginStart);
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_no_autoconnect_status_not_ok() {
+add_task(async function test_no_autoconnect_status_not_ok() {
   let server = sync_httpd_setup();
+  Status.__authManager = Service.identity = new BrowserIDManager();
 
   // Ensure we don't actually try to sync (or log in for that matter).
   function onLoginStart() {
@@ -573,41 +580,36 @@ add_identity_test(this, function* test_no_autoconnect_status_not_ok() {
   }
   Svc.Obs.add("weave:service:login:start", onLoginStart);
 
-  let deferred = Promise.defer();
-  waitForZeroTimer(function () {
-    Svc.Obs.remove("weave:service:login:start", onLoginStart);
-
-    do_check_eq(Status.service, CLIENT_NOT_CONFIGURED);
-    do_check_eq(Status.login, LOGIN_FAILED_NO_USERNAME);
-
-    cleanUpAndGo(server).then(deferred.resolve);
-  });
-
   scheduler.delayedAutoConnect(0);
-  yield deferred.promise;
+  await promiseZeroTimer();
+  Svc.Obs.remove("weave:service:login:start", onLoginStart);
+
+  do_check_eq(Status.service, CLIENT_NOT_CONFIGURED);
+  do_check_eq(Status.login, LOGIN_FAILED_NO_USERNAME);
+
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_autoconnectDelay_pref() {
-  let deferred = Promise.defer();
-  Svc.Obs.add("weave:service:sync:finish", function onSyncFinish() {
-    Svc.Obs.remove("weave:service:sync:finish", onSyncFinish);
-    cleanUpAndGo(server).then(deferred.resolve);
-  });
+add_task(async function test_autoconnectDelay_pref() {
+  enableValidationPrefs();
+
+  let promiseObserved = promiseOneObserver("weave:service:sync:finish");
 
   Svc.Prefs.set("autoconnectDelay", 1);
 
   let server = sync_httpd_setup();
-  yield setUp(server);
+  await setUp(server);
 
   Svc.Obs.notify("weave:service:ready");
 
   // autoconnectDelay pref is multiplied by 1000.
   do_check_eq(scheduler._autoTimer.delay, 1000);
   do_check_eq(Status.service, STATUS_OK);
-  yield deferred.promise;
+  await promiseObserved;
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_idle_adjustSyncInterval() {
+add_task(async function test_idle_adjustSyncInterval() {
   // Confirm defaults.
   do_check_eq(scheduler.idle, false);
 
@@ -618,46 +620,46 @@ add_identity_test(this, function* test_idle_adjustSyncInterval() {
 
   // Multiple devices: switch to idle interval.
   scheduler.idle = false;
-  clientsEngine._store.create({id: "foo", cleartext: "bar"});
+  Svc.Prefs.set("clients.devices.desktop", 1);
+  Svc.Prefs.set("clients.devices.mobile", 1);
   scheduler.updateClientMode();
   scheduler.observe(null, "idle", Svc.Prefs.get("scheduler.idleTime"));
   do_check_eq(scheduler.idle, true);
   do_check_eq(scheduler.syncInterval, scheduler.idleInterval);
 
-  yield cleanUpAndGo();
+  await cleanUpAndGo();
 });
 
-add_identity_test(this, function* test_back_triggersSync() {
+add_task(async function test_back_triggersSync() {
   // Confirm defaults.
   do_check_false(scheduler.idle);
   do_check_eq(Status.backoffInterval, 0);
 
   // Set up: Define 2 clients and put the system in idle.
-  scheduler.numClients = 2;
+  Svc.Prefs.set("clients.devices.desktop", 1);
+  Svc.Prefs.set("clients.devices.mobile", 1);
   scheduler.observe(null, "idle", Svc.Prefs.get("scheduler.idleTime"));
   do_check_true(scheduler.idle);
 
-  let deferred = Promise.defer();
   // We don't actually expect the sync (or the login, for that matter) to
   // succeed. We just want to ensure that it was attempted.
-  Svc.Obs.add("weave:service:login:error", function onLoginError() {
-    Svc.Obs.remove("weave:service:login:error", onLoginError);
-    cleanUpAndGo().then(deferred.resolve);
-  });
+  let promiseObserved = promiseOneObserver("weave:service:login:error");
 
   // Send an 'active' event to trigger sync soonish.
   scheduler.observe(null, "active", Svc.Prefs.get("scheduler.idleTime"));
-  yield deferred.promise;
+  await promiseObserved;
+  await cleanUpAndGo();
 });
 
-add_identity_test(this, function* test_active_triggersSync_observesBackoff() {
+add_task(async function test_active_triggersSync_observesBackoff() {
   // Confirm defaults.
   do_check_false(scheduler.idle);
 
   // Set up: Set backoff, define 2 clients and put the system in idle.
   const BACKOFF = 7337;
   Status.backoffInterval = scheduler.idleInterval + BACKOFF;
-  scheduler.numClients = 2;
+  Svc.Prefs.set("clients.devices.desktop", 1);
+  Svc.Prefs.set("clients.devices.mobile", 1);
   scheduler.observe(null, "idle", Svc.Prefs.get("scheduler.idleTime"));
   do_check_eq(scheduler.idle, true);
 
@@ -666,29 +668,28 @@ add_identity_test(this, function* test_active_triggersSync_observesBackoff() {
   }
   Svc.Obs.add("weave:service:login:start", onLoginStart);
 
-  let deferred = Promise.defer();
-  timer = Utils.namedTimer(function () {
-    Svc.Obs.remove("weave:service:login:start", onLoginStart);
-
-    do_check_true(scheduler.nextSync <= Date.now() + Status.backoffInterval);
-    do_check_eq(scheduler.syncTimer.delay, Status.backoffInterval);
-
-    cleanUpAndGo().then(deferred.resolve);
-  }, IDLE_OBSERVER_BACK_DELAY * 1.5, {}, "timer");
+  let promiseTimer = promiseNamedTimer(IDLE_OBSERVER_BACK_DELAY * 1.5, {}, "timer");
 
   // Send an 'active' event to try to trigger sync soonish.
   scheduler.observe(null, "active", Svc.Prefs.get("scheduler.idleTime"));
-  yield deferred.promise;
+  await promiseTimer;
+  Svc.Obs.remove("weave:service:login:start", onLoginStart);
+
+  do_check_true(scheduler.nextSync <= Date.now() + Status.backoffInterval);
+  do_check_eq(scheduler.syncTimer.delay, Status.backoffInterval);
+
+  await cleanUpAndGo();
 });
 
-add_identity_test(this, function* test_back_debouncing() {
+add_task(async function test_back_debouncing() {
   _("Ensure spurious back-then-idle events, as observed on OS X, don't trigger a sync.");
 
   // Confirm defaults.
   do_check_eq(scheduler.idle, false);
 
   // Set up: Define 2 clients and put the system in idle.
-  scheduler.numClients = 2;
+  Svc.Prefs.set("clients.devices.desktop", 1);
+  Svc.Prefs.set("clients.devices.mobile", 1);
   scheduler.observe(null, "idle", Svc.Prefs.get("scheduler.idleTime"));
   do_check_eq(scheduler.idle, true);
 
@@ -701,30 +702,36 @@ add_identity_test(this, function* test_back_debouncing() {
   scheduler.observe(null, "active", Svc.Prefs.get("scheduler.idleTime"));
   scheduler.observe(null, "idle", Svc.Prefs.get("scheduler.idleTime"));
 
-  let deferred = Promise.defer();
-  timer = Utils.namedTimer(function () {
-    Svc.Obs.remove("weave:service:login:start", onLoginStart);
-    cleanUpAndGo().then(deferred.resolve);
-  }, IDLE_OBSERVER_BACK_DELAY * 1.5, {}, "timer");
-  yield deferred.promise;
+  await promiseNamedTimer(IDLE_OBSERVER_BACK_DELAY * 1.5, {}, "timer");
+  Svc.Obs.remove("weave:service:login:start", onLoginStart);
+  await cleanUpAndGo();
 });
 
-add_identity_test(this, function* test_no_sync_node() {
+add_task(async function test_no_sync_node() {
+  enableValidationPrefs();
+
   // Test when Status.sync == NO_SYNC_NODE_FOUND
   // it is not overwritten on sync:finish
   let server = sync_httpd_setup();
-  yield setUp(server);
+  await setUp(server);
 
-  Service.serverURL = server.baseURI + "/";
+  let oldfc = Service._clusterManager._findCluster;
+  Service._clusterManager._findCluster = () => null;
+  Service.clusterURL = "";
+  try {
+    Service.sync();
+    do_check_eq(Status.sync, NO_SYNC_NODE_FOUND);
+    do_check_eq(scheduler.syncTimer.delay, NO_SYNC_NODE_INTERVAL);
 
-  Service.sync();
-  do_check_eq(Status.sync, NO_SYNC_NODE_FOUND);
-  do_check_eq(scheduler.syncTimer.delay, NO_SYNC_NODE_INTERVAL);
-
-  yield cleanUpAndGo(server);
+    await cleanUpAndGo(server);
+  } finally {
+    Service._clusterManager._findCluster = oldfc;
+  }
 });
 
-add_identity_test(this, function* test_sync_failed_partial_500s() {
+add_task(async function test_sync_failed_partial_500s() {
+  enableValidationPrefs();
+
   _("Test a 5xx status calls handleSyncError.");
   scheduler._syncErrors = MAX_ERROR_COUNT_BEFORE_BACKOFF;
   let server = sync_httpd_setup();
@@ -735,7 +742,7 @@ add_identity_test(this, function* test_sync_failed_partial_500s() {
 
   do_check_eq(Status.sync, SYNC_SUCCEEDED);
 
-  do_check_true(yield setUp(server));
+  do_check_true(await setUp(server));
 
   Service.sync();
 
@@ -748,10 +755,12 @@ add_identity_test(this, function* test_sync_failed_partial_500s() {
   do_check_true(scheduler.nextSync <= (Date.now() + maxInterval));
   do_check_true(scheduler.syncTimer.delay <= maxInterval);
 
-  yield cleanUpAndGo(server);
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_sync_failed_partial_400s() {
+add_task(async function test_sync_failed_partial_400s() {
+  enableValidationPrefs();
+
   _("Test a non-5xx status doesn't call handleSyncError.");
   scheduler._syncErrors = MAX_ERROR_COUNT_BEFORE_BACKOFF;
   let server = sync_httpd_setup();
@@ -761,11 +770,13 @@ add_identity_test(this, function* test_sync_failed_partial_400s() {
   engine.exception = {status: 400};
 
   // Have multiple devices for an active interval.
-  clientsEngine._store.create({id: "foo", cleartext: "bar"});
+  clientsEngine._store.create(
+    { id: "foo", cleartext: { os: "mobile", version: "0.01", type: "desktop" } }
+  );
 
   do_check_eq(Status.sync, SYNC_SUCCEEDED);
 
-  do_check_true(yield setUp(server));
+  do_check_true(await setUp(server));
 
   Service.sync();
 
@@ -778,19 +789,21 @@ add_identity_test(this, function* test_sync_failed_partial_400s() {
   do_check_true(scheduler.nextSync <= (Date.now() + scheduler.activeInterval));
   do_check_true(scheduler.syncTimer.delay <= scheduler.activeInterval);
 
-  yield cleanUpAndGo(server);
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_sync_X_Weave_Backoff() {
+add_task(async function test_sync_X_Weave_Backoff() {
+  enableValidationPrefs();
+
   let server = sync_httpd_setup();
-  yield setUp(server);
+  await setUp(server);
 
   // Use an odd value on purpose so that it doesn't happen to coincide with one
   // of the sync intervals.
   const BACKOFF = 7337;
 
   // Extend info/collections so that we can put it into server maintenance mode.
-  const INFO_COLLECTIONS = "/1.1/johndoe/info/collections";
+  const INFO_COLLECTIONS = "/1.1/johndoe@mozilla.com/info/collections";
   let infoColl = server._handler._overridePaths[INFO_COLLECTIONS];
   let serverBackoff = false;
   function infoCollWithBackoff(request, response) {
@@ -803,7 +816,9 @@ add_identity_test(this, function* test_sync_X_Weave_Backoff() {
 
   // Pretend we have two clients so that the regular sync interval is
   // sufficiently low.
-  clientsEngine._store.create({id: "foo", cleartext: "bar"});
+  clientsEngine._store.create(
+    { id: "foo", cleartext: { os: "mobile", version: "0.01", type: "desktop" } }
+  );
   let rec = clientsEngine._store.createRecord("foo", "clients");
   rec.encrypt(Service.collectionKeys.keyForCollection("clients"));
   rec.upload(Service.resource(clientsEngine.engineURL + rec.id));
@@ -824,28 +839,30 @@ add_identity_test(this, function* test_sync_X_Weave_Backoff() {
   Service.sync();
 
   do_check_true(Status.backoffInterval >= BACKOFF * 1000);
-  // Allowing 1 second worth of of leeway between when Status.minimumNextSync
+  // Allowing 3 seconds worth of of leeway between when Status.minimumNextSync
   // was set and when this line gets executed.
-  let minimumExpectedDelay = (BACKOFF - 1) * 1000;
+  let minimumExpectedDelay = (BACKOFF - 3) * 1000;
   do_check_true(Status.minimumNextSync >= Date.now() + minimumExpectedDelay);
 
   // Verify that the next sync is actually going to wait that long.
   do_check_true(scheduler.nextSync >= Date.now() + minimumExpectedDelay);
   do_check_true(scheduler.syncTimer.delay >= minimumExpectedDelay);
 
-  yield cleanUpAndGo(server);
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_sync_503_Retry_After() {
+add_task(async function test_sync_503_Retry_After() {
+  enableValidationPrefs();
+
   let server = sync_httpd_setup();
-  yield setUp(server);
+  await setUp(server);
 
   // Use an odd value on purpose so that it doesn't happen to coincide with one
   // of the sync intervals.
   const BACKOFF = 7337;
 
   // Extend info/collections so that we can put it into server maintenance mode.
-  const INFO_COLLECTIONS = "/1.1/johndoe/info/collections";
+  const INFO_COLLECTIONS = "/1.1/johndoe@mozilla.com/info/collections";
   let infoColl = server._handler._overridePaths[INFO_COLLECTIONS];
   let serverMaintenance = false;
   function infoCollWithMaintenance(request, response) {
@@ -860,7 +877,9 @@ add_identity_test(this, function* test_sync_503_Retry_After() {
 
   // Pretend we have two clients so that the regular sync interval is
   // sufficiently low.
-  clientsEngine._store.create({id: "foo", cleartext: "bar"});
+  clientsEngine._store.create(
+    { id: "foo", cleartext: { os: "mobile", version: "0.01", type: "desktop" } }
+  );
   let rec = clientsEngine._store.createRecord("foo", "clients");
   rec.encrypt(Service.collectionKeys.keyForCollection("clients"));
   rec.upload(Service.resource(clientsEngine.engineURL + rec.id));
@@ -883,42 +902,25 @@ add_identity_test(this, function* test_sync_503_Retry_After() {
 
   do_check_true(Status.enforceBackoff);
   do_check_true(Status.backoffInterval >= BACKOFF * 1000);
-  // Allowing 1 second worth of of leeway between when Status.minimumNextSync
+  // Allowing 3 seconds worth of of leeway between when Status.minimumNextSync
   // was set and when this line gets executed.
-  let minimumExpectedDelay = (BACKOFF - 1) * 1000;
+  let minimumExpectedDelay = (BACKOFF - 3) * 1000;
   do_check_true(Status.minimumNextSync >= Date.now() + minimumExpectedDelay);
 
   // Verify that the next sync is actually going to wait that long.
   do_check_true(scheduler.nextSync >= Date.now() + minimumExpectedDelay);
   do_check_true(scheduler.syncTimer.delay >= minimumExpectedDelay);
 
-  yield cleanUpAndGo(server);
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_loginError_recoverable_reschedules() {
+add_task(async function test_loginError_recoverable_reschedules() {
   _("Verify that a recoverable login error schedules a new sync.");
-  yield configureIdentity({username: "johndoe"});
-  Service.serverURL = "http://localhost:1234/";
-  Service.clusterURL = Service.serverURL;
-  Service.persistLogin();
+  await configureIdentity({username: "johndoe@mozilla.com"});
+  Service.clusterURL = "http://localhost:1234/";
   Status.resetSync(); // reset Status.login
 
-  let deferred = Promise.defer();
-  Svc.Obs.add("weave:service:login:error", function onLoginError() {
-    Svc.Obs.remove("weave:service:login:error", onLoginError);
-    Utils.nextTick(function aLittleBitAfterLoginError() {
-      do_check_eq(Status.login, LOGIN_FAILED_NETWORK_ERROR);
-
-      let expectedNextSync = Date.now() + scheduler.syncInterval;
-      do_check_true(scheduler.nextSync > Date.now());
-      do_check_true(scheduler.nextSync <= expectedNextSync);
-      do_check_true(scheduler.syncTimer.delay > 0);
-      do_check_true(scheduler.syncTimer.delay <= scheduler.syncInterval);
-
-      Svc.Obs.remove("weave:service:sync:start", onSyncStart);
-      cleanUpAndGo().then(deferred.resolve);
-    });
-  });
+  let promiseObserved = promiseOneObserver("weave:service:login:error");
 
   // Let's set it up so that a sync is overdue, both in terms of previously
   // scheduled syncs and the global score. We still do not expect an immediate
@@ -936,45 +938,33 @@ add_identity_test(this, function* test_loginError_recoverable_reschedules() {
   do_check_eq(Status.login, LOGIN_SUCCEEDED);
 
   scheduler.scheduleNextSync(0);
-  yield deferred.promise;
+  await promiseObserved;
+  await promiseNextTick();
+
+  do_check_eq(Status.login, LOGIN_FAILED_NETWORK_ERROR);
+
+  let expectedNextSync = Date.now() + scheduler.syncInterval;
+  do_check_true(scheduler.nextSync > Date.now());
+  do_check_true(scheduler.nextSync <= expectedNextSync);
+  do_check_true(scheduler.syncTimer.delay > 0);
+  do_check_true(scheduler.syncTimer.delay <= scheduler.syncInterval);
+
+  Svc.Obs.remove("weave:service:sync:start", onSyncStart);
+  await cleanUpAndGo()
 });
 
-add_identity_test(this, function* test_loginError_fatal_clearsTriggers() {
+add_task(async function test_loginError_fatal_clearsTriggers() {
   _("Verify that a fatal login error clears sync triggers.");
-  yield configureIdentity({username: "johndoe"});
+  await configureIdentity({username: "johndoe@mozilla.com"});
 
   let server = httpd_setup({
-    "/1.1/johndoe/info/collections": httpd_handler(401, "Unauthorized")
+    "/1.1/johndoe@mozilla.com/info/collections": httpd_handler(401, "Unauthorized")
   });
 
-  Service.serverURL = server.baseURI + "/";
-  Service.clusterURL = Service.serverURL;
-  Service.persistLogin();
+  Service.clusterURL = server.baseURI + "/";
   Status.resetSync(); // reset Status.login
 
-  let deferred = Promise.defer();
-  Svc.Obs.add("weave:service:login:error", function onLoginError() {
-    Svc.Obs.remove("weave:service:login:error", onLoginError);
-    Utils.nextTick(function aLittleBitAfterLoginError() {
-
-      if (isConfiguredWithLegacyIdentity()) {
-        // for the "legacy" identity, a 401 on info/collections means the
-        // password is wrong, so we enter a "login rejected" state.
-        do_check_eq(Status.login, LOGIN_FAILED_LOGIN_REJECTED);
-
-        do_check_eq(scheduler.nextSync, 0);
-        do_check_eq(scheduler.syncTimer, null);
-      } else {
-        // For the FxA identity, a 401 on info/collections means a transient
-        // error, probably due to an inability to fetch a token.
-        do_check_eq(Status.login, LOGIN_FAILED_NETWORK_ERROR);
-        // syncs should still be scheduled.
-        do_check_true(scheduler.nextSync > Date.now());
-        do_check_true(scheduler.syncTimer.delay > 0);
-      }
-      cleanUpAndGo(server).then(deferred.resolve);
-    });
-  });
+  let promiseObserved = promiseOneObserver("weave:service:login:error");
 
   // Sanity check.
   do_check_eq(scheduler.nextSync, 0);
@@ -983,10 +973,20 @@ add_identity_test(this, function* test_loginError_fatal_clearsTriggers() {
   do_check_eq(Status.login, LOGIN_SUCCEEDED);
 
   scheduler.scheduleNextSync(0);
-  yield deferred.promise;
+  await promiseObserved;
+  await promiseNextTick();
+
+  // For the FxA identity, a 401 on info/collections means a transient
+  // error, probably due to an inability to fetch a token.
+  do_check_eq(Status.login, LOGIN_FAILED_NETWORK_ERROR);
+  // syncs should still be scheduled.
+  do_check_true(scheduler.nextSync > Date.now());
+  do_check_true(scheduler.syncTimer.delay > 0);
+
+  await cleanUpAndGo(server);
 });
 
-add_identity_test(this, function* test_proper_interval_on_only_failing() {
+add_task(async function test_proper_interval_on_only_failing() {
   _("Ensure proper behavior when only failed records are applied.");
 
   // If an engine reports that no records succeeded, we shouldn't decrease the
@@ -1003,13 +1003,8 @@ add_identity_test(this, function* test_proper_interval_on_only_failing() {
     reconciled: 0
   });
 
-  let deferred = Promise.defer();
-  Utils.nextTick(function() {
-    scheduler.adjustSyncInterval();
-    do_check_false(scheduler.hasIncomingItems);
-    do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
-
-    deferred.resolve();
-  });
-  yield deferred.promise;
+  await promiseNextTick();
+  scheduler.adjustSyncInterval();
+  do_check_false(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
 });
